// 自动发送tiktok 发送消息

(function () {
  let messageTool = new MessageTool({
    nameSpace: "autoSendMessage",
  });

  // 发送tiktok 私信
  messageTool.listenChromeMessage((request, sendResponse) => {
    let { topic, params } = request;

    // 自动发送消息
    if (topic === "autoContact") {
      handleAutoContact(params, sendResponse);
      return true;
    }

    // 获取聊天历史
    if (topic === "getMessageHistory") {
      handleGetMessageHistory(params, sendResponse);
      return true;
    }

    // 获取未读小红点达人
    if (topic === "unReadKolList") {
      handleUnReadKolList(params, sendResponse);
      return true;
    }

    // 获取未读小红点达人的聊天历史
    if (topic === "getUnReadKolChatHistory") {
      handleGetUnreadChatHistory(params, sendResponse);
      return true;
    }
    // 获取达人聊天状态
    if (topic === "getKolChatStatus") {
      handleGetKolChatStatus(params, sendResponse);
      return true;
    }
  });

  const timerObj = {
    time: 1000,
    num: 0,
    max: 10,
  };

  // 自动发送消息
  let timer1 = 0;

  function handleAutoContact(params, sendResponse) {
    let { message, tiktokMessageTabId } = params;
    clearTimer(timer1, () => {
      timer1 = 0;
    });
    timer1 = setInterval(() => {
      let dom = document.querySelector(".public-DraftEditor-content");
      if (dom) {
        clearTimer(timer1, () => {
          timer1 = 0;
        });

        // postmessage 通知发送消息
        setTimeout(() => {
          messageTool.sendPostMessage(
            {
              topic: "autoContact",
              params: {
                message,
              },
            },
            "https://www.tiktok.com"
          );
        }, 1000);

        // 定时五秒响应chrome消息
        setTimeout(() => {
          // 默认发送成功
          messageTool.sendChromeResponse(
            {
              topic: "autoContact",
              params: {
                tiktokMessageTabId,
              },
            },
            sendResponse
          );
        }, 5000);
      }
    }, timerObj.time * 1);
  }

  // 获取聊天历史
  let timer2 = 0;

  function handleGetMessageHistory(params, sendResponse) {
    let { uniqueId, tiktokMessageTabId } = params;
    clearTimer(timer2, () => {
      timer2 = 0;
    });
    timer2 = setInterval(() => {
      let dom = document.querySelector('[data-e2e="chat-item"]');
      if (dom) {
        clearTimer(timer2, () => {
          timer2 = 0;
        });

        //延迟三秒获取聊天历史
        // setTimeout(() => {
        let chatHistory = [];

        // 获取聊天历史
        let chatItems = Array.from(
          document.querySelectorAll('[data-e2e="chat-item"]')
        );
        // 如果没有聊天数组
        if (chatItems.lenght === 0) {
          messageTool.sendChromeResponse(
            {
              topic: "getMessageHistory",
              params: {
                uniqueId,
                chatHistory,
                tiktokMessageTabId,
              },
            },
            sendResponse
          );
          return;
        }

        // 反转数组并截取最新的25条数据
        chatItems = chatItems.reverse().slice(0, 25);
        chatItems.forEach((item) => {
          // css-bn40ho-DivMessageContainer 为kol 类名 css-veqxhm-DivMessageContainer 为user 类名
          // 根据图片类名区分
          let role = item.querySelector(`[href='/@${uniqueId}']`) ? 1 : 2;
          if (item.querySelector(".css-1rdxtjl-PText")) {
            chatHistory.push({
              role,
              content:
                item.querySelector(".css-1rdxtjl-PText")?.innerText || "",
            });
          }
        });

        // 返回聊天历史
        messageTool.sendChromeResponse(
          {
            topic: "getMessageHistory",
            params: {
              uniqueId,
              chatHistory,
              tiktokMessageTabId,
            },
          },
          sendResponse
        );
        // }, 3000);
      } else {
        timerObj.num += 1;
        if (timerObj.num >= timerObj.max) {
          clearTimer(timer2, () => {
            timer2 = 0;
          });

          messageTool.sendChromeResponse(
            {
              topic: "getMessageHistory",
              params: {
                uniqueId,
                chatHistory: [],
                tiktokMessageTabId,
              },
            },
            sendResponse
          );
        }
      }
    }, timerObj.time * 1);
  }

  // 获取获取未读的小红点达人
  let timer3 = 0;
  let unReadDomList = [];
  let randomList = [];

  function handleUnReadKolList(params, sendResponse) {
    let { tiktokMessageTabId } = params;

    unReadDomList = [];
    randomList = [];
    clearTimer(timer3, () => {
      timer3 = 0;
    });
    timer3 = setInterval(() => {
      // 判断当前是否包含聊天历史
      // 如果当前聊天界面对应达人
      let currentMessage = document.querySelector(".css-khdl9x-DivItemWrapper");
      if (currentMessage) {
        let currentChatHistroy = Array.from(
          document.querySelectorAll('[data-e2e="chat-item"]')
        );

        if (currentChatHistroy.length) {
          let chatDom = currentChatHistroy[currentChatHistroy.length - 1];
          let uniqueId =
            document.querySelector('[data-e2e="chat-uniqueid"]') || "";
          // 判断是否有uniqueId值
          if (uniqueId) {
            uniqueId = uniqueId.innerText.startsWith("@")
              ? uniqueId.innerText.slice(1, uniqueId.innerText.length)
              : "";
          }

          if (chatDom.querySelector(`[href='/@${uniqueId}']`)) {
            let random = Math.random().toString(32).slice(2, 16);
            unReadDomList.push({
              dom: currentMessage,
              random,
            });
            randomList.push(random);
          }
        }
      }

      // 获取未读小红点达人
      let skeletonDoms = document.querySelectorAll(
        ".css-4x0rp9-DivSkeletonContainer"
      ).length;

      if (!skeletonDoms) {
        // 未读数据
        let doms =
          document.querySelectorAll(".css-62oc03-SpanNewMessage") || [];

        // 获取父级dom
        doms.forEach((dom) => {
          let random = Math.random().toString(32).slice(2, 16);
          unReadDomList.push({
            random,
            dom: dom.parentNode,
          });
          randomList.push(random);
        });

        messageTool.sendChromeResponse(
          {
            topic: "unReadKolList",
            params: {
              tiktokMessageTabId,
              randomList,
            },
          },
          sendResponse
        );

        clearTimer(timer3, () => {
          timer3 = 0;
        });
      }
    }, timerObj.time * 1);
  }

  // 获取未读达人聊天历史
  function handleGetUnreadChatHistory(params, sendResponse) {
    try {
      let { domRandom } = params;
      let i = unReadDomList.findIndex((item) => item.random === domRandom);
      unReadDomList[i].dom.click();

      setTimeout(() => {
        let uniqueId =
          document.querySelector('[data-e2e="chat-uniqueid"]') || "";
        // 判断是否有uniqueId值
        if (uniqueId) {
          uniqueId = uniqueId.innerText.startsWith("@")
            ? uniqueId.innerText.slice(1, uniqueId.innerText.length)
            : "";
        }

        let nickName =
          document.querySelector('[data-e2e="chat-nickname"]')?.innerText || "";

        // 获取聊天历史
        let chatItems = Array.from(
          document.querySelectorAll('[data-e2e="chat-item"]')
        );
        // 如果没有数组
        if (chatItems.lenght === 0) {
          messageTool.sendChromeResponse(
            {
              topic: "getUnReadKolChatHistory",
              params: {
                status: "success",
                chatHistory: [],
                uniqueId,
                nickName,
              },
            },
            sendResponse
          );
          return;
        }

        // 反转数组并截取最新的25条数据
        chatItems = chatItems.reverse().slice(0, 25);
        let chatHistory = [];
        chatItems.forEach((item) => {
          // 根据图片类名区分
          let role = item.querySelector(`[href='/@${uniqueId}']`) ? 1 : 2;
          if (item.querySelector(".css-1rdxtjl-PText")) {
            chatHistory.push({
              role,
              content:
                item.querySelector(".css-1rdxtjl-PText")?.innerText || "",
            });
          }
        });
        messageTool.sendChromeResponse(
          {
            topic: "getUnReadKolChatHistory",
            params: {
              status: "success",
              chatHistory: chatHistory.reverse(),
              uniqueId,
              nickName,
            },
          },
          sendResponse
        );
      }, 3000);
    } catch {
      messageTool.sendChromeResponse(
        {
          topic: "getUnReadKolChatHistory",
          params: {
            status: "fail",
            chatHistory: [],
            uniqueId: "",
            nickName: "",
          },
        },
        sendResponse
      );
    }
  }

  function handleGetKolChatStatus(params, sendResponse) {
    let { tiktokMessageList } = params;
    let statusText =
      document.querySelector(".css-1rspuj3-DivSendFailTip")?.innerText || "";
    let status = 0; // 1为隐私设置 2为提示切换账号

    let chatItems = Array.from(
      document.querySelectorAll('[data-e2e="chat-item"]')
    );
    // 判断发送消息是否有错误
    let chatWarning = null;
    if (chatItems.length) {
      chatWarning =
        chatItems[chatItems.length - 1].querySelector('[data-e2e="dm-warning"]')
          ?.innerHTML || null;
      if (chatWarning) status = 4;
    }
    if (tiktokMessageList.includes(statusText)) {
      status = 1;
    } else if (statusText) {
      status = 2;
    }
    messageTool.sendChromeResponse(
      {
        topic: "getKolChatStatus",
        params: {
          status,
        },
      },
      sendResponse
    );
  }
  // 清除定时器
  function clearTimer(timer, callback) {
    if (timer) {
      clearInterval(timer);
    }
    timerObj.num = 0;
    callback && callback();
  }
})();
