console.log("popup.js执行");

/************************** chrome message 相关逻辑 start ***************************************/

// 监听chrome 消息
chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
  // 容错处理后期全部改为JSON格式

  let { topic, message } = request;
  console.log(topic, message, "popup.js");

  if (topic === "sendStorage" && message.from === "service_worker") {
    sendResponse({
      topic: "sendStorage",
      message: {
        status: "success",
        from: "popup",
      },
    });
    handleSendStorage(message);
    return true;
  }

  return true; // Indicate that response will be sent asynchronously
});

// 通过postMessage 发送消息
function handleSendStorage(message) {
  try {
    let { data } = message;
    sendPostMessage({
      topic: "sendStorage",
      params: { ...data, from: "popup" },
    });
  } catch {
    console.log("error");
  }
}

/**
 * 发送chrome 消息
 * @param {*} params
 * @returns
 */
function sendChromeMessage(params = { topic: "", message: "" }) {
  return new Promise((resolve, reject) => {
    let { topic, message } = params;

    if (!topic || !message) {
      reject("topic or message is empty");
    }
    chrome.runtime.sendMessage(
      {
        topic,
        params: message,
      },
      (response) => {
        if (chrome.runtime.lastError) {
          console.error(chrome.runtime.lastError);
          reject();
        } else {
          if (!response) return;

          let { topic, message } = response;
          if (topic && message) {
            resolve({
              topic,
              message: message || {},
            });
          }
        }
      }
    );
  });
}

/************************** chrome message 相关逻辑 end ***************************************/

/************************** postMessage 相关逻辑 start ***************************************/
// 监听postMessage
window.onload = () => {
  window.addEventListener("message", function (event) {
    if (event.data && event.data.topic) {
      let { topic, params } = event.data;
      // 接受search Again
      if (topic === "searchAgain") {
        handleSearchAgain(params);
        return;
      }
      // 验证copilot 是否打开
      if (topic === "checkInfluenbase") {
        handleCheckCopilot(params, event);
        return;
      }
    }
  });
};

// 点击发送按钮
function handleSearchAgain(data) {
  try {
    sendChromeMessage({
      topic: "searchAgain",
      message: {
        data,
        from: "popup",
      },
    });
  } catch {}
}

// 验证登录
function handleCheckCopilot(data, event) {
  // 如果300毫秒内不出现 验证弹窗 则认为验证失败
  try {
    let sendFlag = false;
    const timeout = setTimeout(() => {
      event.source.postMessage({
        topic: "checkInfluenbaseIsWorkflow",
        params: {
          isOpen: false,
          isWorkflow: false,
          isLogin: false,
        },
      });
      sendFlag = true;
      throw "如果500毫秒内不出现 验证弹窗 则认为验证失败";
    }, 500);
    sendChromeMessage({
      topic: "checkInfluenbase",
      message: {
        from: "popup",
        userId: data.userId,
      },
    })
      .then(({ topic, message }) => {
        clearTimeout(timeout);
        if (
          topic === "checkInfluenbase" &&
          message.from === "Influenbase" &&
          !sendFlag
        )
          event.source.postMessage({
            topic: "checkInfluenbaseIsWorkflow",
            params: {
              isOpen: message.isOpen,
              isWorkflow: message.isWorkflow,
              isLogin: message.isLogin,
            },
          });
      })
      .catch((err) => {
        clearTimeout(timeout);
        if (!sendFlag) {
          event.source.postMessage({
            topic: "checkInfluenbaseIsWorkflow",
            params: {
              isOpen: false,
              isWorkflow: false,
              isLogin: false,
            },
          });
        }
      });
  } catch (e) {}
}

/**
 * 发送postMessage
 * @param {*} data
 * @param {*} target
 */
function sendPostMessage(data, target = "*") {
  if (!data.topic) {
    return console.error("请传入topic");
  }
  window.postMessage(data, target);
}
/************************** postMessage 相关逻辑 end ***************************************/
