// 点击图标打开侧边栏
chrome.sidePanel
  .setPanelBehavior({
    openPanelOnActionClick: true,
  })
  .catch((error) => console.error(error));

// 谷歌登录验证
chrome.action.onClicked.addListener(() => {
  chrome.identity.getAuthToken(
    {
      interactive: true,
    },
    function (token) {
      if (chrome.runtime.lastError) {
        return;
      }

      fetch("https://www.googleapis.com/oauth2/v1/userinfo?alt=json", {
        headers: {
          Authorization: "Bearer " + token,
        },
      })
        .then((response) => response.json())
        .then((userInfo) => {
          console.log("User Info:", userInfo);
        })
        .catch((error) => console.error("Error fetching user info:", error));
    }
  );
});

// 监听的方法
const topicObj = {
  // 获取chrome 信息
  getChromeInfo: async (request, sendResponse) => {
    // 获取插件生成的windowId
    let windowId = await new Promise((resolve, reject) => {
      chrome.windows.getLastFocused({}, function (window) {
        resolve(window.id);
      });
    });

    // 获取windowId
    sendResponse({
      topic: "getChromeInfo",
      params: {
        from: "serviceWorker",
        windowId,
      },
    });

    // // 获取当前tabId 并且同步windowId 和chrome信息
    // chrome.tabs.query({}, function (tabs) {
    //   let tabId = tabs[0].id;
    //   chrome.tabs.sendMessage(
    //     tabId,
    //     {
    //       topic: "getChromeInfo",
    //       params: {
    //         from: "serviceWorker",
    //         windowId,
    //       },
    //     },
    //     (response) => {
    //       sendResponse(response);
    //     }
    //   );
    // });
  },

  // 打开历史记录
  openHistory: async (request, sendResponse) => {
    // 打开历史记录弹窗
    handleOpenHistory(request.params, sendResponse);
    return true;
  },

  // 验证tiktok 是否登录
  isTiktokLogin: async (request, sendResponse) => {
    try {
      let { topic, params } = request;
      let tiktokId = await changeTabId(params?.tiktokMessageTabId || null, {
        url: "https://www.tiktok.com/",
        active: true,
        windowId: params.windowId,
      });

      // 发送消息
      await sendTabMessage(
        tiktokId,
        {
          topic: "isTiktokLogin",
          params: {
            tiktokId,
          },
        },
        sendResponse
      );
    } catch {
      sendInfluenbaseSendResponse(
        {
          topic: "isTiktokLogin",
          params: {
            isLogin: false,
            iserror: true,
          },
        },
        sendResponse
      );
    }
  },

  // 根据hashTag 获取达人
  getKolData: async (request, sendResponse) => {
    try {
      let { params } = request;
      let _searchKolTabId = await changeTabId(params.searchKolTabId, {
        url: `https://www.tiktok.com/search?q=${params.hashTag}`,
        active: true,
        windowId: params.windowId,
      });

      // 创建弹窗
      handleCreateModel(_searchKolTabId, params.language);

      // 发送消息
      await sendTabMessage(
        _searchKolTabId,
        {
          topic: "getKolData",
          params: {
            searchKolTabId: _searchKolTabId,
            language: params.language,
          },
        },
        sendResponse
      );
    } catch {
      sendInfluenbaseSendResponse(
        {
          topic: "getKolData",
          params: {
            iserror: true,
            kolData: [],
          },
        },
        sendResponse
      );
    }
  },

  // 通过hashTag 获取更多的达人详情
  getKolMoreData: async (request, sendResponse) => {
    try {
      let { params } = request;
      let { searchKolTabId, windowId } = params;

      let _searchKolTabId = await changeTabId(searchKolTabId, {
        active: true,
        windowId,
      });

      await sendTabMessage(
        Number(params.searchKolTabId),
        {
          topic: "getKolMoreData",
          params: {
            searchKolTabId: _searchKolTabId,
          },
        },
        sendResponse
      );
    } catch {
      sendInfluenbaseSendResponse(
        {
          topic: "getKolMoreData",
          params: {
            iserror: true,
            kolData: [],
          },
        },
        sendResponse
      );
    }
  },

  // 获取达人主页详情数据
  getKolDetail: async (request, sendResponse) => {
    try {
      let { params } = request;
      let { uniqueId, tiktokDetailTabId, windowId, language } = params;
      let _tiktokDetailTabId = await changeTabId(tiktokDetailTabId, {
        url: `https://www.tiktok.com/@${uniqueId}`,
        active: true,
        windowId,
      });

      // 创建弹窗
      handleCreateModel(_tiktokDetailTabId, language);

      // 创建分析弹窗
      setTimeout(() => {
        sendTabMessage(_tiktokDetailTabId, {
          topic: "createAnalyzeModel",
          params: {
            tabId: _tiktokDetailTabId,
            language,
          },
        });
      }, 0);

      // 获取kol detail
      await sendTabMessage(
        _tiktokDetailTabId,
        {
          topic: "getKolDetail",
          params: {
            tiktokDetailTabId: _tiktokDetailTabId,
            uniqueId,
            language,
          },
        },
        sendResponse
      );
    } catch {
      sendInfluenbaseSendResponse(
        {
          topic: "getKolDetail",
          params: {
            isError: false,
            tiktokDetailTabId: 0,
            uniqueId: "",
            nikename: "",
            messageId: "",
            followerCountShow: "",
            likesShow: "",
            description: "",
            link: "",
            avatar: "",
            region: "",
            covers: "",
          },
        },
        sendResponse
      );
    }
  },

  // 关闭达人详情页
  closeDetailTab: async (request, sendResponse) => {
    try {
      let { params } = request;
      let { tiktokDetailTabId, searchKolTabId, windowId } = params;

      // 关闭窗口
      await closeTabId(tiktokDetailTabId);
      // 激活列表窗口
      let _searchKolTabId = await changeTabId(searchKolTabId, {
        active: true,
        windowId,
      });

      // 返回消息
      sendResponse({
        topoc: "closeDetailTab",
        params: {
          from: "serviceWorker",
        },
      });
    } catch {
      sendInfluenbaseSendResponse(
        {
          topic: "closeDetailTab",
          params: {
            isError: false,
          },
        },
        sendResponse
      );
    }
  },

  // 自动发送消息
  autoContact: async (request, sendResponse) => {
    try {
      let { params } = request;
      let {
        uniqueId,
        messageId,
        tiktokMessageTabId,
        message,
        language,
        windowId,
      } = params;

      let _tiktokMessageTabId = await changeTabId(tiktokMessageTabId, {
        url: `https://www.tiktok.com/messages?lang=en&u=${messageId}`,
        active: true,
        windowId,
      });

      // 创建弹窗
      handleCreateModel(_tiktokMessageTabId, language);

      // 自动发送消息
      await sendTabMessage(
        _tiktokMessageTabId,
        {
          topic: "autoContact",
          params: {
            tiktokMessageTabId: _tiktokMessageTabId,
            uniqueId,
            message,
            language,
          },
        },
        sendResponse
      );
    } catch {
      sendInfluenbaseSendResponse(
        {
          topic: "autoContact",
          params: {
            isError: false,
          },
        },
        sendResponse
      );
    }
  },

  // 获取聊天历史
  getMessageHistory: async (request, sendResponse) => {
    try {
      let { params } = request;
      let { uniqueId, messageId, tiktokMessageTabId, language, windowId } =
        params;

      let _tiktokMessageTabId = await changeTabId(tiktokMessageTabId, {
        url: `https://www.tiktok.com/messages?lang=en&u=${messageId}`,
        active: true,
        windowId,
      });

      // 创建弹窗
      handleCreateModel(_tiktokMessageTabId, language);

      // 自动发送消息
      await sendTabMessage(
        _tiktokMessageTabId,
        {
          topic: "getMessageHistory",
          params: {
            tiktokMessageTabId: _tiktokMessageTabId,
            uniqueId,
            language,
          },
        },
        sendResponse
      );
    } catch {
      sendInfluenbaseSendResponse(
        {
          topic: "getMessageHistory",
          params: {
            isError: false,
          },
        },
        sendResponse
      );
    }
  },

  // 获取未读小红点达人消息
  unReadKolList: async (request, sendResponse) => {
    try {
      let { params } = request;
      let { tiktokMessageTabId, windowId } = params;

      let _tiktokMessageTabId = await changeTabId(tiktokMessageTabId, {
        active: true,
        windowId,
      });

      // 创建弹窗
      handleCreateModel(_tiktokMessageTabId);

      await sendTabMessage(
        _tiktokMessageTabId,
        {
          topic: "unReadKolList",
          params: {
            tiktokMessageTabId: _tiktokMessageTabId,
          },
        },
        sendResponse
      );
    } catch {
      sendInfluenbaseSendResponse(
        {
          topic: "unReadKolList",
          params: {
            isError: false,
          },
        },
        sendResponse
      );
    }
  },

  // 获取未读小红点达人聊天历史
  getUnReadKolChatHistory: (request, sendResponse) => {
    try {
      let { params } = request;
      let { domRandom, tiktokMessageTabId } = params;

      sendTabMessage(
        Number(tiktokMessageTabId),
        {
          topic: "getUnReadKolChatHistory",
          params: {
            domRandom,
          },
        },
        sendResponse
      );
    } catch {
      sendInfluenbaseSendResponse(
        {
          topic: "getUnReadKolChatHistory",
          params: {
            isError: false,
          },
        },
        sendResponse
      );
    }
  },
  // 获取达人顶部聊天状态
  getKolChatStatus: (request, sendResponse) => {
    let { params } = request;
    sendTabMessage(
      Number(params.tiktokMessageTabId),
      {
        topic: "getKolChatStatus",
        params: {
          tiktokMessageList: params.tiktokMessageList,
        },
      },
      sendResponse
    );
  },

  // 关闭弹窗
  closeModel: (request, sendResponse) => {
    try {
      let { params } = request;
      let { tabIds } = params;
      tabIds.forEach((tabId) => {
        if (tabId) {
          sendTabMessage(tabId, {
            topic: "closeModel",
            params: {
              tabId,
            },
          });
        }
      });
    } catch {
      sendInfluenbaseSendResponse(
        {
          topic: "closeModel",
          params: {
            isError: false,
          },
        },
        sendResponse
      );
    }
  },

  // 关闭历史记录弹窗
  closeHistory: (request, sendResponse) => {
    let { params } = request;
    let { historyTabIdList } = params;
    historyTabIdList.forEach((tabId) => {
      if (tabId) {
        chrome.tabs.remove(tabId);
      }
    });
    sendResponse({
      topic: "closeHistory",
    });
  },

  // 打开指定链接
  openLink: async (request, sendResponse) => {
    let { params } = request;
    let { link } = params;

    let tabId = await changeTabId(null, {
      url: link,
      active: true,
    });

    sendResponse({
      topic: "openLink",
      params: {
        tabId,
      },
    });
  },
};

// chrome 监听消息
chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
  let { topic, params } = request;

  if (topic !== "checkPluginStatus") {
    console.log(topic, params, "service worker", topicObj[topic]);
  }

  // if (params.from !== "Influenbase") {
  //   sendResponse({
  //     topic: "error",
  //     params: {
  //       message: "消息来源错误",
  //       status: "error",
  //     },
  //   });
  //   return true;
  // }

  topicObj[topic] && topicObj[topic](request, sendResponse);
  return true;
});

// 是否还有存在的tab
function hasTabId(tabId, params) {
  return new Promise((resolve, reject) => {
    chrome.tabs.query(
      {
        ...params,
      },
      function (tabs) {
        // 可以遍历 tabs 数组，处理每个选项卡对象
        let tabIndex = tabs.findIndex((item) => item.id === Number(tabId));

        if (tabIndex === -1) {
          resolve({
            tabItem: null,
            tabIndex,
          });
        } else {
          resolve({
            tabItem: tabs[tabIndex],
            tabIndex,
          });
        }
      }
    );
  });
}

// 如果有id 则更新tab 否则创建tab
function changeTabId(tabId, params) {
  return new Promise(async (resolve, reject) => {
    // 校验参数
    let _params = {};
    // 指定windowId 窗口
    if (params.windowId) {
      _params.windowId = params.windowId;
    }

    const { tabIndex, tabItem } = await hasTabId(tabId, _params);
    console.log(tabId, tabIndex, "标签页 与 标签索引");

    if (tabItem) {
      // 如果有windowId 则删除windowId
      if (params.windowId) {
        delete params.windowId;
      }

      chrome.tabs.update(Number(tabId), params, function (newTab) {
        if (chrome.runtime.lastError) {
          console.error(chrome.runtime.lastError);
          reject();
        } else {
          // 如果没有指定链接这直接返回resolve
          if (!params.url) {
            resolve(Number(tabId));
            return;
          }

          const callback = (tabId, changeInfo, tab) => {
            if (
              changeInfo.status === "complete" &&
              tabId === Number(newTab.id)
            ) {
              resolve(Number(newTab.id));
              chrome.tabs.onUpdated.removeListener(callback);
            }
          };

          chrome.tabs.onUpdated.addListener(callback);
        }
      });
    } else {
      chrome.tabs.create(params, function (newTab) {
        if (chrome.runtime.lastError) {
          console.error(chrome.runtime.lastError);
          reject();
        } else {
          const callback = (tabId, changeInfo, tab) => {
            if (
              changeInfo.status === "complete" &&
              tabId === Number(newTab.id)
            ) {
              resolve(Number(newTab.id));
              chrome.tabs.onUpdated.removeListener(callback);
            }
          };

          chrome.tabs.onUpdated.addListener(callback);
        }
      });
    }
  });
}

// 关闭指定标签页
function closeTabId(tabId, params) {
  return new Promise(async (resolve, reject) => {
    let { tabItem, tabIndex } = await hasTabId(Number(tabId));
    console.log(tabItem, tabId, "关闭标签页");
    if (tabItem) {
      chrome.tabs.remove(Number(tabId), function () {
        if (chrome.runtime.lastError) {
          console.error(chrome.runtime.lastError);
          reject();
        } else {
          resolve();
        }
      });
    } else {
      resolve();
    }
  });
}

// 发送消息到指定tabId
function sendTabMessage(tabId, request, sendResponse = undefined) {
  return new Promise((resolve, reject) => {
    // 校验参数
    request.params && (request.params.from = "serviceWorker");

    chrome.tabs.sendMessage(
      Number(tabId),
      {
        ...request,
      },
      (response) => {
        if (chrome.runtime.lastError) {
          console.error(chrome.runtime.lastError);
          reject();
        } else {
          // 如果需要传参
          sendResponse && sendResponse(response);
          return resolve(response);
        }
      }
    );
  });
}

// 打开历史记录
function handleOpenHistory(message, sendResponse) {
  try {
    let { url, systemConfig, userInfoConfig } = message;

    chrome.windows.create(
      {
        url, // 替换为你想要打开的页面URL
        width: 1200, // 设置窗口的宽度
        height: 1000, // 设置窗口的高度
        left: 100, //新窗口距离左侧位置
        top: 100, // 新窗口距离右侧位置
      },
      (window) => {
        // 发送标签页消息
        chrome.tabs.query(
          {
            active: true,
            windowId: window.id,
          },
          function (tabs) {
            // 已经打开历史消息弹窗
            sendResponse({
              topic: "openHistory",
              params: {
                data: [],
                historyWindowId: window.id,
                historyTabId: tabs[0].id, // 当前标签页id
                status: "success",
              },
            });

            // 监听 tab 加载完成事件
            chrome.tabs.onUpdated.addListener((tabId, changeInfo, tab) => {
              if (changeInfo.status === "complete" && tabId === tabs[0].id) {
                chrome.tabs.sendMessage(
                  tabs[0].id,
                  {
                    topic: "sendStorage",
                    message: {
                      data: {
                        systemConfig,
                        userInfoConfig,
                      },
                      status: "success",
                      from: "service_worker",
                    },
                  },
                  (response) => {
                    if (chrome.runtime.lastError) {
                      console.error(chrome.runtime.lastError);
                    } else {
                      // 发送响应到 search.vue
                      return true;
                    }
                  }
                );
              }
            });
          }
        );
      }
    );
  } catch {
    sendResponse({
      topic: "openHistory",
      message: {
        data: [],
        historyWindowId: 0,
        status: "fail",
      },
    });
  }
}

// 直接返回到sendResponse
function sendInfluenbaseSendResponse(
  options = {
    topic: "",
    params: {},
  },
  sendResponse
) {
  // 校验参数
  options.params && (options.params.from = "serviceWorker");
  sendResponse({
    ...options,
  });
  return true;
}

// 创建弹窗
function handleCreateModel(tabId, language = "zh_CN") {
  sendTabMessage(tabId, {
    topic: "createModel",
    params: {
      tabId,
      language,
    },
  }).then(({ topic, params }) => {
    console.log(topic, params, "创建弹窗");
  });
}
