// chrome 窗体信息
const chromeInfo = localStorage.getItem("Influenbase-chromeInfo")
  ? JSON.parse(localStorage.getItem("Influenbase-chromeInfo"))
  : {
      windowId: 0, // 当前的windowId
      tabId: 0, // 当前的tabId
    };

// 消息工具
// 接受chrome 消息
// 发送消息
// 主动发送chrome 消息
// 接受postMessage
// 发送postMessage

class MessageTool {
  nameSpace = ""; // 命名空间

  constructor(options = { nameSpace: "" }) {
    let { nameSpace } = options;
    this.nameSpace = options.nameSpace;
    console.log(this.nameSpace, "已加载");
  }

  getNameSpace() {
    return this.nameSpace;
  }

  setNameSpace(nameSpace) {
    this.nameSpace = nameSpace;
  }

  // 接受chrome 消息
  listenChromeMessage(callback) {
    if (chrome && chrome.runtime) {
      chrome?.runtime?.onMessage?.addListener(
        (request, sender, sendResponse) => {
          try {
            let { topic, params } = request;

            console.log(topic, params, "消息接受");

            if (!params.from) return new Error("chrome 消息来源必传");
            if (!topic) return new Error("chrome 消息topic 不存在");
            // if (params.windowId && params.windowId !== chromeInfo.windowID)
            //   return new Error("windowId 不一致");

            console.log(topic, params, this.nameSpace);

            // 回调函数
            callback(request, sendResponse);
            return true;
          } catch (err) {
            sendResponse({
              topic: "Error",
              params: {
                status: "Error",
                message: err,
              },
            });
          }
        }
      );
    } else {
      new Error("不支持 chrome 语法");
    }
  }

  // 响应chrome sendMessage 消息通信
  sendChromeResponse(
    options = { status: "success", message: "" },
    sendResponse
  ) {
    let { topic, params } = options;
    if (!params?.from) {
      params.from = this.nameSpace;
    }
    console.log("消息拦截", topic, params);
    sendResponse({
      topic,
      params,
    });
    return true;
  }

  // 主动发送chrome 消息
  sendChromeMessage(
    options = {
      topic: "",
      params: {},
    }
  ) {
    return new Promise((resolve, reject) => {
      let { topic, params } = options;

      if (!topic) return new Error("topic 不存在");

      let _params = Object.assign(
        {
          // windowId: chromeInfo.windowID,
          from: this.nameSpace,
        },
        params
      );
      // 发送消息
      chrome.runtime.sendMessage(
        {
          topic,
          params: _params,
        },
        (response) => {
          if (chrome.runtime.lastError) {
            console.log(chrome.runtime.lastError);
            reject();
          } else {
            resolve(response);
          }
        }
      );
    });
  }

  // 接受postMessage
  listenPostMessage(callback) {
    console.log("开始监听postMessage");
    window.addEventListener("message", function (event) {
      console.log("🚩", event);

      if (event.data && event.data.topic) {
        let { topic, params } = event.data;

        if (!params.from) return new Error("postMessage 消息来源必传");
        if (!topic) return new Error("postMessage 消息topic 不存在");
        if (params.windowId && params.windowId !== chromeInfo.windowId)
          return new Error("windowId 不一致");

        console.log(topic, "接受 postMessage消息");
        callback(event.data, event);
      }
    });
  }

  // 发送postMessage
  sendPostMessage(
    options = {
      topic: "",
      params: {},
    },
    event = "*"
  ) {
    let { topic, params } = options;
    if (!topic) return new Error("topic 不存在");
    if (!params) return new Error("params 不存在");
    if (!event) return new Error("event 不存在");
    console.log("发送消息", topic, event);
    // 如果 发送event 是地址 则直接发送postMessage
    if (typeof event === "string") {
      if (!event) return new Error("指定postMessage 发送地址不存在");

      window.postMessage(
        {
          topic,
          params,
        },
        event
      );
    } else {
      // 发送event 消息
      event.source.postMessage({
        topic,
        params,
      });
    }
  }
}

// 生成一个消息实例 监听信息
const MessageToolInstance = new MessageTool({
  nameSpace: "messageTool",
});

// 监听chrome 消息
MessageToolInstance.listenChromeMessage((request, sendResponse) => {
  let { topic, params } = request;

  // 设置chrome
  if (topic === "getChromeInfo") {
    let { windowId } = params;
    chromeInfo.windowId = windowId;
    localStorage.setItem("Influenbase-chromeInfo", JSON.stringify(chromeInfo));

    // 回复消息
    MessageToolInstance.sendChromeResponse(
      {
        topic: "getChromeInfo",
        params: {
          windowId: chromeInfo.windowId,
        },
      },
      sendResponse
    );

    // 定时轮询
    messageLoopObj.init();
  }

  // 创建tabId
  if (topic === "createTab") {
    let { tabId } = params;
    chromeInfo.tabId = tabId;
    localStorage.setItem("Influenbase-chromeInfo", JSON.stringify(chromeInfo));
  }

  // 删除tab上的蒙层
  if (topic === "delTabModel") {
    let { tabId } = params;
    let dom = document.querySelector(`bp-model--${tabId}`);
    if (dom) {
      document.removeChild(dom);
    }
  }
});

// 消息轮询 检测插件是否打开
const messageLoopObj = {
  timer: 0, // 定时器，
  clearTimer() {
    if (this.timer) {
      clearInterval(this.timer);
      this.timer = 0;
    }
  },
  init() {
    this.clearTimer();
    this.timer = setInterval(() => {
      chrome.runtime.sendMessage(
        {
          topic: "checkPluginStatus",
          params: {
            windowId: this.windowId,
            from: "content",
          },
        },
        (response) => {
          if (chrome.runtime.lastError) {
            console.log(chrome.runtime.lastError, "checkPluginStatus");
            // 关闭定时器
            this.clearTimer();
            // 清除弹窗
            this.closeModel();
            return;
          }
          if (!response) {
            // 关闭定时器
            this.clearTimer();
            // 清除弹窗
            this.closeModel();
          }
        }
      );
    }, 1000);
  },

  closeModel() {
    let dom = document.querySelector("bp-model");
    if (dom) {
      dom.remove();
    }
  },
};
